const { spawn, spawnSync, exec } = require('child_process')
const chalk = require('chalk')

const { TEMPLATES } = require('./question')
const { isYarn, CURR_DIR } = require('./helper')
const { updateTslintConfig } = require('./updateTslintConfig')
const { updatePackageJsonFile } = require('./updatePackageJsonFile')
const { copyFileToApp } = require('./copyFiles')
const { installCreateReactApp } = require('./installCreateReactApp')
const { installNecessaryProject } = require('./installNecessaryProject')

module.exports.createTypescriptProject = ({ name, template, hasExistApp }) => {
  const TARGET_PATH = `${CURR_DIR}`

  if (!name) {
    name = 'reactjs-ts-app'
  }
  name = `${TARGET_PATH}/${name}`

  if (!TEMPLATES.includes(template)) {
    template = 'typescript'
  }

  if (!hasExistApp) {
    // create react ts app

    // install create react app if not exist and create new app
    installCreateReactApp({ isYarn, projectName: name })
  }

  // install necessary package
  installNecessaryProject({ isYarn })

  // update tslint config
  console.log(chalk.green(`🥳 Config base URL ...`))
  updateTslintConfig()

  // Config scripts
  console.log(chalk.green(`🥳 Config scripts ...`))
  updatePackageJsonFile()

  // Run prepare
  console.log(chalk.green(`🥳 Run prepare ...`))
  spawnSync(isYarn ? 'yarn' : 'npm', ['run prepare'], {
    shell: true,
    stdio: 'inherit'
  })

  // copy files
  copyFileToApp({
    CURR_DIR,
    TARGET_PATH
  })
}
